@@ -37,9 +37,10 @@ logSys = getLogger(__name__)
 class FilterReader(DefinitionInitConfigReader):
 
 	_configOpts = {
+		"usedns": ["string", None],
 		"prefregex": ["string", None],
 		"ignoreregex": ["string", None],
-		"failregex": ["string", ""],
+		"failregex": ["string", None],
 		"maxlines": ["int", None],
 		"datepattern": ["string", None],
 		"journalmatch": ["string", None],
@@ -57,6 +58,11 @@ class FilterReader(DefinitionInitConfigReader):
 		opts = self.getCombined()
 		if not len(opts):
 			return stream
+		return FilterReader._fillStream(stream, opts, self._jailName)
+
+	@staticmethod
+	def _fillStream(stream, opts, jailName):
+		prio0idx = 0
 		for opt, value in opts.items():
 			if opt in ("failregex", "ignoreregex"):
 				if value is None: continue
@@ -66,21 +72,22 @@ class FilterReader(DefinitionInitConfigReader):
 					if regex != '':
 						multi.append(regex)
 				if len(multi) > 1:
-					stream.append(["multi-set", self._jailName, "add" + opt, multi])
+					stream.append(["multi-set", jailName, "add" + opt, multi])
 				elif len(multi):
-					stream.append(["set", self._jailName, "add" + opt, multi[0]])
-			elif opt in ('maxlines', 'prefregex'):
-				# Be sure we set this options first.
-				stream.insert(0, ["set", self._jailName, opt, value])
+					stream.append(["set", jailName, "add" + opt, multi[0]])
+			elif opt in ('usedns', 'maxlines', 'prefregex'):
+				# Be sure we set this options first, and usedns is before all regex(s).
+				stream.insert(0 if opt == 'usedns' else prio0idx,
+					["set", jailName, opt, value])
+				prio0idx += 1
 			elif opt in ('datepattern'):
-				stream.append(["set", self._jailName, opt, value])
-			# Do not send a command if the match is empty.
+				stream.append(["set", jailName, opt, value])
 			elif opt == 'journalmatch':
+				# Do not send a command if the match is empty.
 				if value is None: continue
 				for match in value.split("\n"):
 					if match == '': continue
 					stream.append(
-						["set", self._jailName, "addjournalmatch"] +
-                        shlex.split(match))
+						["set", jailName, "addjournalmatch"] + shlex.split(match))
 		return stream
 		
