@@ -61,7 +61,7 @@ class Ticket(object):
 					self._data[k] = v
 		if ticket:
 			# ticket available - copy whole information from ticket:
-			self.__dict__.update(i for i in iter(ticket.__dict__.items()) if i[0] in self.__dict__)
+			self.__dict__.update(i for i in ticket.__dict__.items() if i[0] in self.__dict__)
 
 	def __str__(self):
 		return "%s: ip=%s time=%s #attempts=%d matches=%r" % \
@@ -135,7 +135,13 @@ class Ticket(object):
 		return self._data['failures']
 
 	def setMatches(self, matches):
-		self._data['matches'] = matches or []
+		if matches:
+			self._data['matches'] = matches
+		else:
+			try:
+				del self._data['matches']
+			except KeyError:
+				pass
 
 	def getMatches(self):
 		return [(line if not isinstance(line, (list, tuple)) else "".join(line)) \
@@ -171,7 +177,7 @@ class Ticket(object):
 		elif len(args) == 2:
 			self._data.update((args,))
 		elif len(args) > 2:
-			self._data.update((k,v) for k,v in list(zip(*[iter(args)]*2)))
+			self._data.update((k,v) for k,v in zip(*[iter(args)]*2))
 		if len(argv):
 			self._data.update(argv)
 		# filter (delete) None values:
@@ -200,6 +206,13 @@ class Ticket(object):
 		# return single value of data:
 		return self._data.get(key, default)
 
+	@property
+	def banEpoch(self):
+		return getattr(self, '_banEpoch', 0)
+	@banEpoch.setter
+	def banEpoch(self, value):
+		self._banEpoch = value
+
 
 class FailTicket(Ticket):
 
